-- MUX za index i vrednost Rd registra
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.constants.all;

entity reg_rd_mux is
	port 
	(
		in_op_type : in type_of_instruction_t; -- koji je tip instrukcije
		
		in_rd : in reg_t; -- koji je trenutni index Rd
		in_rd_data : in ulv32_t; -- vrednost rd
		in_mem_data : in ulv32_t; -- vrednost iz memorije
		in_LINK_data : in ulv32_t; -- vrednost link registra
		
		out_rd : out reg_t; -- izlaz za index
		out_data : out ulv32_t -- izlazni podaci
	);
end entity;

architecture rtl of reg_rd_mux is
    
begin

	with in_op_type select
		out_rd <= 	LINK_REG when B_BL, -- Ako je BL Upisujemo u Link registar (nece se upisati u slucaju kad je B jer Controller stavlja rd_we na '0')
					in_rd when others; -- U svim drugim slucajevima ide obicna vrednost

	with in_op_type select
		out_data <= in_rd_data when DP_R, -- rezultat alu ili swap operacije
					in_rd_data when DP_I, -- rezultat alu ili swap operacije
					in_mem_data when RWM, -- RWM trazi da se vrednost iz memorije prvo upise u rd pa onda da se radi alu operacija
					in_LINK_data when B_BL, -- Ako je BL Upisujemo u Link registar (nece se upisati u slucaju kad je B jer Controller stavlja rd_we na '0')
					in_mem_data when L_S, -- Ako je load onda se iz memorije upisuje u rd
					ZERO_ULV32_T when others; -- nebitno nam je
  
end architecture;
