-- Implementacija carry look ahead addera
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.adder_defs.all;
use work.constants.all;

entity adder_carry_look_ahead is
	port
	(
		in_s : in adder_in_signals; -- ulazni podaci
		out_s : out adder_out_signals -- izlazni podaci
	);
end entity;

architecture rtl of adder_carry_look_ahead is

-- pogledati na http://en.wikipedia.org/wiki/Carry-lookahead_adder

signal h_sum : ulv32_t;
signal carry_generate : ulv32_t;
signal carry_propagate : ulv32_t;
signal carry_in_internal : ulv32_t;

begin
	h_sum <= in_s.operator_1 xor in_s.operator_2;
	carry_generate <= in_s.operator_1 and in_s.operator_2;
	carry_propagate <= in_s.operator_1 or in_s.operator_2;
	process (all)
	begin
		carry_in_internal(ulv32_t'low + 1) <= carry_generate(ulv32_t'low) or (carry_propagate(ulv32_t'low) and in_s.csr_c);
		inst: for i in ulv32_t'low + 1 to ulv32_t'high - 1 loop
			carry_in_internal(i+ulv32_t'low + 1) <= carry_generate(i) or (carry_propagate(i) and carry_in_internal(i));
		end loop;
		out_s.csr_c <= carry_generate(ulv32_t'high) or (carry_propagate(ulv32_t'high) and carry_in_internal(ulv32_t'high));
	end process;

	out_s.result(ulv32_t'low) <= h_sum(ulv32_t'low) xor in_s.csr_c;
	out_s.result(ulv32_t'high downto ulv32_t'low + 1) <= h_sum(ulv32_t'high downto ulv32_t'low + 1) xor carry_in_internal(ulv32_t'high downto ulv32_t'low + 1);
	out_s.csr_z <= '1' when (out_s.result = ZERO_ULV32_T) else '0';
	out_s.csr_v <= out_s.csr_c xor carry_in_internal(ulv32_t'high);
	out_s.csr_n <= out_s.result(ulv32_t'high); 
end architecture;  
