-- adder uradjen kao funkcija umesto da se koristi full adders
-- ima 2 arhitekture jedina razlika je sto jedna radi nor redukciju dok druga radi obican equals
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.adder_defs.all;
use work.constants.all;

entity adder_ripple_carry_function is
	port
	(
		in_s : in adder_in_signals; -- ulazni podaci
		out_s : out adder_out_signals -- izlazni podaci
	);
end entity;

architecture equals of adder_ripple_carry_function is

begin

	addition : process (all)
	constant bit_width : integer:= ulv32_t'high; -- potrebna nam je velicina ulaznog podatka
	alias a: ulv32_t is in_s.operator_1; -- da se lakse pise
	alias b: ulv32_t is in_s.operator_2; -- da se lakse pise
	variable result: ulv32_t; -- rezultat
	variable carry_curent : bit_t; -- carry za n
	variable carry_previous : bit_t; -- carry za n-1

	begin
		carry_curent := in_s.csr_c; -- stavljamo pocetni carry
		for i in 0 to (bit_width - 1) loop
			result(i) := carry_curent xor a(i) xor b(i);
			carry_curent := (carry_curent and a(i)) or (carry_curent and b(i)) or (a(i) and b(i));
		end loop;
		carry_previous := carry_curent;
		result(bit_width) := carry_curent xor a(bit_width) xor b(bit_width);
		carry_curent := (carry_curent and a(bit_width)) or (carry_curent and b(bit_width)) or (a(bit_width) and b(bit_width));
		out_s.result <= result;
		out_s.csr_c <= carry_curent;
		out_s.csr_v <= carry_curent xor carry_previous;
		out_s.csr_n <= result(bit_width);
		if (result = ZERO_ULV32_T) then
			out_s.csr_z <= '1';
		else
			out_s.csr_z <= '0';
		end if;
	end process;

end architecture;

architecture nor_reduce of adder_ripple_carry_function is

begin

	addition : process (all)
	constant bit_width : integer:= ulv32_t'high; -- potrebna nam je velicina ulaznog podatka
	alias a: ulv32_t is in_s.operator_1; -- da se lakse pise
	alias b: ulv32_t is in_s.operator_2; -- da se lakse pise
	variable result: ulv32_t; -- rezultat
	variable carry_curent : bit_t; -- carry za n
	variable carry_previous : bit_t; -- carry za n-1
	variable zero : bit_t; -- da li je nula
	begin
		zero := '0'; -- postavljamo nulu
		carry_curent := in_s.csr_c; -- stavljamo pocetni carry
		for i in 0 to (bit_width - 1) loop
			result(i) := carry_curent xor a(i) xor b(i);
			carry_curent := (carry_curent and a(i)) or (carry_curent and b(i)) or (a(i) and b(i));
			zero := zero or result(i);
		end loop;
		zero := zero or result(bit_width);
		carry_previous := carry_curent;
		result(bit_width) := carry_curent xor a(bit_width) xor b(bit_width);
		carry_curent := (carry_curent and a(bit_width)) or (carry_curent and b(bit_width)) or (a(bit_width) and b(bit_width));
		out_s.result <= result;
		out_s.csr_c <= carry_curent;
		out_s.csr_v <= carry_curent xor carry_previous;
		out_s.csr_n <= result(bit_width);
		out_s.csr_z <= not zero; -- da bi bila nor redukcija radimo not
	end process;

end architecture;
