-- adder sa medju prenosom nule
library ieee;
use ieee.std_logic_1164.all;

use work.full_adder_defs.all;
use work.adder_defs.all;
use work.constants.all;

entity adder_ripple_carry_with_zero_propagation is
	port
	(
		in_s : in adder_in_signals; -- ulazni podaci
		out_s : out adder_out_signals -- izlazni podaci
	);
end entity;

architecture nor_reduction of adder_ripple_carry_with_zero_propagation is

constant operand_length : natural := in_s.operator_1'high; -- potrebno zbog generisanja odgovarajuceg broja adder

subtype ulv31_t is ulv_t(operand_length - 1 downto 0); -- posto nam nije potreban poslednji bit (jer ga direktno upisujemo u rezultat) ovaj podtip je za 1 bit kraci

signal c : ulv31_t; -- carry bit koji se prenosi
signal z : ulv31_t; -- nula koja se prenosi

-- ulazni podaci za full adder
type array_of_in_signals is array (ulv32_t'range) of full_adder_1_bit_with_zero_in_signals;
signal f_a_in : array_of_in_signals;

-- izlazni podaci za full adder
type array_of_out_signals is array (ulv32_t'range) of full_adder_1_bit_with_zero_out_signals;
signal f_a_out : array_of_out_signals;

begin 

	Full_Adders:
	for i in 0 to operand_length 
	generate
	begin
		-- osnovno vezivanje
		f_a_in(i).a <= in_s.operator_1(i); 
		f_a_in(i).b <= in_s.operator_2(i);
	 
		LOW_BIT: if (i = 0) -- u nulti bit se ubacuje C bit iz adera (ALU) i stavlja se nula na '0'
		generate
			f_a_in(i).c <= in_s.csr_c;
			f_a_in(i).z <= '0';
			F_A_0 : full_adder_1_bit_with_zero port map (f_a_in(i), f_a_out(i));
			c(i) <= f_a_out(i).carry;
			z(i) <= f_a_out(i).zero;
		end generate LOW_BIT;

		MIDDLE_BITS: if (i > 0) and (i < operand_length)  -- 1 to n - 2 uzimaju vrednosti iz prethodnika
		generate
			f_a_in(i).c <= c(i - 1);
			f_a_in(i).z <= z(i - 1);
			F_A_I : full_adder_1_bit_with_zero port map (f_a_in(i), f_a_out(i));
			c(i) <= f_a_out(i).carry;
			z(i) <= f_a_out(i).zero;
		end generate MIDDLE_BITS;
	 
		TOP_BIT: if (i = operand_length) -- n-1 smesta vrednosti u izlaz
		generate
			f_a_in(i).c <= c(i - 1);
			f_a_in(i).z <= z(i - 1);
			F_A_N : full_adder_1_bit_with_zero port map (f_a_in(i), f_a_out(i));
			out_s.csr_c <= f_a_out(operand_length).carry;
			out_s.csr_z <= not f_a_out(operand_length).zero; -- posto se radi or u svakoj prethodnoj jedinici da bi se dobio nor result potreban nam je not
		end generate TOP_BIT;
    
		out_s.result(i) <= f_a_out(i).sum; -- sum u rezultat
   end generate;
	
	out_s.csr_n <= out_s.result(operand_length);
	out_s.csr_v <= out_s.csr_c xor c(operand_length - 1); -- overflow se nalazi ako se uradi c(32)xor c(31)

end architecture;
