-- genericka arhitektura

architecture alu_generic_arch of alu is
  subtype u32_t is unsigned(BUS_WIDTH - 1 downto 0); -- unsigned velicine 32 bit
  subtype u33_t is unsigned(BUS_WIDTH downto 0); -- koristi se za carry (32 bit + 32 bit => 32 bit ili 33 bit ako ima prekoracenja)
  signal alu_op_result : u32_t; -- Rezultat operacije
  
  signal csr_n : bit_t; -- Rezultat je negativan
  signal csr_z : bit_t; -- Rezultat je nula
  signal csr_c : bit_t; -- Postoji prenos u sledeci razred
  signal csr_v : bit_t; -- Doslo je do prekoracenja

begin  
  out_s.result <= ulv32_t(alu_op_result); -- Vracamo rezultat
  
  out_s.csr_n <= csr_n; -- Vrednost n bita
  out_s.csr_z <= csr_z; -- Smestamo vrenost z bita
  out_s.csr_c <= csr_c; -- Vrednost c bita
  out_s.csr_v <= csr_v; -- Smestamo vrenost v bita

  Perform_ALU_Function : process (all)
  variable a : u33_t; -- variabla za prvi operand
  variable b : u33_t; -- variabla za drugi operand
  variable ab : u33_t; -- a +- b
  variable abc : u33_t;-- a +- b +- in_csr_c
  variable ab_v : bit_t; -- prekoracenje u slucaju a +- b
  variable abc_v : bit_t; -- prekoracenje u slucaju ab +- 1
    
  begin
	-- postavljamo na nulu da bi sprecili latch
	alu_op_result <= u32_t(ZERO_ULV32_T);
	csr_n <= '0';
	csr_z <= '0';
	csr_c <= '0';
	csr_v <= '0';
	a := '0' & u32_t(in_s.operator_1);
	b := '0' & u32_t(in_s.operator_2);
	
	case in_s.alu_op is -- Proveravamo koja je operacija
	
	  when ALU_FN_ADD => -- ADD
		 ab := a + b;
		 csr_c <= ab(BUS_WIDTH);
		 alu_op_result <= ab(BUS_WIDTH - 1 downto 0);
		 if ((a(BUS_WIDTH - 1) = b(BUS_WIDTH - 1)) and (ab(BUS_WIDTH - 1) = not b(BUS_WIDTH - 1))) then
			csr_v <= '1';
		 else
			csr_v <= '0';
		 end if;
		 csr_n <= ab(BUS_WIDTH - 1);
		 if (ab(BUS_WIDTH - 1 downto 0) = u32_t(ZERO_ULV32_T)) then
			csr_z <= '1';
		 else
			csr_z <= '0';
		 end if;

	  when ALU_FN_ADDC => -- ADDC = ADD + C bit
		 ab := a + b;
		 if ((a(BUS_WIDTH - 1) = b(BUS_WIDTH - 1)) and (ab(BUS_WIDTH - 1) = not b(BUS_WIDTH - 1))) then
			ab_v := '1';
		 else
			ab_v := '0';
		 end if;
		 abc := ab + u33_t'(0 => in_s.csr_c, others => '0');
		 csr_c <= abc(BUS_WIDTH);
		 alu_op_result <= abc(BUS_WIDTH - 1 downto 0);
		 if ((ab(BUS_WIDTH - 1) = '0') and (abc(BUS_WIDTH - 1) = '1')) then
			abc_v := '1';
		 else
			abc_v := '0';
		 end if;
		 csr_v <= ab_v or abc_v;
		 csr_n <= abc(BUS_WIDTH - 1);
		 if (abc(BUS_WIDTH - 1 downto 0) = u32_t(ZERO_ULV32_T)) then
			csr_z <= '1';
		 else
			csr_z <= '0';
		 end if;

	  when ALU_FN_SUB_OR_CMP => -- SUB OR CMP
		 ab := a - b;
		 csr_c <= ab(BUS_WIDTH);
		 alu_op_result <= ab(BUS_WIDTH - 1 downto 0);
		 if ((a(BUS_WIDTH - 1) = not b(BUS_WIDTH - 1)) and (ab(BUS_WIDTH - 1) = b(BUS_WIDTH - 1))) then
			csr_v <= '1';
		 else
			csr_v <= '0';
		 end if;
		 csr_n <= ab(BUS_WIDTH - 1);
		 if (ab(BUS_WIDTH - 1 downto 0) = u32_t(ZERO_ULV32_T)) then
			csr_z <= '1';
		 else
			csr_z <= '0';
		 end if;

	  when ALU_FN_SUBC => -- SUB = SUB - C bit
		 ab := a - b;
		 if ((a(BUS_WIDTH - 1) = not b(BUS_WIDTH - 1)) and (ab(BUS_WIDTH - 1) = b(BUS_WIDTH - 1))) then
			ab_v := '1';
		 else
			ab_v := '0';
		 end if;
		 abc := ab - u33_t'(0 => in_s.csr_c, others => '0');
		 csr_c <= abc(BUS_WIDTH);
		 alu_op_result <= abc(BUS_WIDTH - 1 downto 0);
		 if ((ab(BUS_WIDTH - 1) = '1') and (abc(BUS_WIDTH - 1) = '0')) then
			abc_v := '1';
		 else
			abc_v := '0';
		 end if;
		 csr_v <= ab_v or abc_v;
		 csr_n <= abc(BUS_WIDTH - 1);if (abc(BUS_WIDTH - 1 downto 0) = u32_t(ZERO_ULV32_T)) then
			csr_z <= '1';
		 else
			csr_z <= '0';
		 end if;

	  when ALU_FN_AND => -- AND
		 ab := a and b;
		 csr_c <= '0';
		 alu_op_result <= ab(BUS_WIDTH - 1 downto 0);
		 csr_v <= '0';
		 csr_n <= ab(BUS_WIDTH - 1);
		 if (ab(BUS_WIDTH - 1 downto 0) = u32_t(ZERO_ULV32_T)) then
			csr_z <= '1';
		 else
			csr_z <= '0';
		 end if;
		 
	  when ALU_FN_NOT => --NOT
		 ab := not a;
		 alu_op_result <= ab(BUS_WIDTH - 1 downto 0);
		 csr_v <= '0';
		 csr_c <= '0';
		 csr_n <= ab(BUS_WIDTH - 1);
		 if (ab(BUS_WIDTH - 1 downto 0) = u32_t(ZERO_ULV32_T)) then
			csr_z <= '1';
		 else
			csr_z <= '0';
		 end if;
		 
	 when ALU_FN_MOV => --MOV
		 alu_op_result <= u32_t(in_s.operator_2);
		 csr_v <= '0';
		 csr_c <= '0';
		 csr_n <= '0';
		 csr_z <= '0';

	  when others =>
		 null;

	end case;
  end process;

end architecture;
