library ieee;
use ieee.std_logic_1164.all;

use work.constants.all;

package control_unit_defs is

type decoder_out_signals is record

	-- Za sve
	op_type : op_type_t; -- Tip instrukcije koji se vraca iz Decodera
	-- DP R, DP I, RMW
	op_code : op_code_t; -- Code instrukcije koji se vraca iz Decodera
	rn : reg_t; -- Rn registar
	rd : reg_t; -- Rd registar
	s_bit : bit_t; -- S (Signed/Unsigned) Bit
	
	-- DP R, RMW
	rm : reg_t; -- Rm registar
	
	-- DP R
	rs : reg_t; -- Rs registar
	shift_or_rot : bit_t; -- F (Shift/Rot) bit
	shift_rot_dir : u2_t; -- Shift/rot smer
	
	-- DP I
	immediate : immed_t; -- Immediate vrednost DP I
	
	-- L/S
	load_or_store : bit_t; -- L (Load/store) Bit
	
	-- B/BL
	jump_condition : u2_t; -- Condition code
	branch_or_branch_and_link : bit_t; -- L (Jump/Link) Bit
	jump_offset : offs_t; -- Offset
	
end record;

type controller_in_signals is record
	op_type : op_type_t; -- Tip instrukcije koji se vraca iz Decodera
	op_code : op_code_t; -- Code instrukcije koji se vraca iz Decodera
	load_or_store : bit_t; -- L (Load/store) Bit
	branch_or_branch_and_link : bit_t; -- L (Jump/Link) Bit
end record;

type controller_out_signals is record
	op_type : type_of_instruction_t; -- signal koji oznacava tip instrukcije
	alu_op : alu_fn_t; -- signal koji se salje u ALU sa tipom instrukcije koja se izvrsava
	
	ir_we : bit_t; -- da li se upisuje u Instrucition Register
	
	rd_we : bit_t; -- da li se upisuje u rd
	rm_we : bit_t; -- da li se upisuje u rm
	
	mem_we : bit_t; -- da li se upisuje u memoriju
	
	swap_instruction : bit_t; -- da li je SWAP
	csr_we : bit_t; -- Da li se vrsi upis u CSR
	PC_we : bit_t; -- da li se vrsi upis vrednosti PC u R15
end record;

component decoder is
	port
	(
		signal in_instruction : in ulv32_t; -- Instrukcija koja dolazi u Decoder

		signal out_s : out decoder_out_signals -- razbijena instrukcija
	);
end component;

component controller is
	port
	(
		in_clk : in std_ulogic; -- ulazni clock
		in_rst : in std_ulogic; -- reset
		in_s : in controller_in_signals; -- ulazni signali
		out_s : out controller_out_signals -- izlazni signali
	);
end component;

component link_and_pc_counter is
	port 
	(
		op_type : in type_of_instruction_t; -- koji je tip instrukcije
		in_PC_data : in ulv32_t; --trenutna vrednost PC
		in_OE_data : in ulv32_t; -- prosireni offset u slucaju da je B/BL
		in_jump_condition : in u2_t; -- da li je B/BL
		in_branch_or_branch_and_link : in bit_t; -- da li je B ili BL
		in_csr_flag_n : in bit_t; -- n flag iz CSR
		in_csr_flag_z : in bit_t; -- z flag iz CSR
		in_csr_flag_c : in bit_t; -- c flag iz CSR
		in_csr_flag_v : in bit_t; -- v flag iz CSR
		
		out_LINK_data : out ulv32_t; -- izlazni podaci za LINK registar
		out_PC_data : out ulv32_t -- izlazna vrednost za PC
	);
end component;

end package;
