-- Fajl za upravljanje sa PC i LINK
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.constants.all;

entity link_and_pc_counter is
	port 
	(
		op_type : in type_of_instruction_t; -- koji je tip instrukcije
		in_PC_data : in ulv32_t; --trenutna vrednost PC
		in_OE_data : in ulv32_t; -- prosireni offset u slucaju da je B/BL
		in_jump_condition : in u2_t; -- da li je B/BL
		in_branch_or_branch_and_link : in bit_t; -- da li je B ili BL
		in_csr_flag_n : in bit_t; -- n flag iz CSR
		in_csr_flag_z : in bit_t; -- z flag iz CSR
		in_csr_flag_c : in bit_t; -- c flag iz CSR
		in_csr_flag_v : in bit_t; -- v flag iz CSR
		
		out_LINK_data : out ulv32_t; -- izlazni podaci za LINK registar
		out_PC_data : out ulv32_t -- izlazna vrednost za PC
	);
end entity;

architecture rtl of link_and_pc_counter is
	
begin

	Update_SPEC_REG: process (all) is
	variable next_address : ulv32_t; -- sledeca adresa na koju se postavlja PC
	variable jump_address : ulv32_t; -- adresa u slucaju B/BL

	begin
		next_address := ulv32_t(unsigned(in_PC_data) + 1); -- povecavamo trenutnu vrednost PC sa 1 => next_address = PC + 1
		if (to_integer(next_address) > (INST_MEM_SIZE - 1)) then
			next_address := ZERO_ULV32_T; -- Ako je to izvan memorije resetujemo se na 0 instrukciju
		end if;
		out_LINK_data <= next_address; -- LINK postavljamo na next_address

		if (op_type = B_BL) then
			jump_address := ulv32_t(signed(next_address) + signed(in_OE_data)); -- jump = (PC + 1) + offset

			case (in_jump_condition) is
				-- zasto se koriste ovi flagovi pogledati u http://community.arm.com/groups/processors/blog/2010/07/16/condition-codes-1-condition-flags-and-codes
				when B_BL_EQ => -- x == y
					if (in_csr_flag_z = '1') then
						next_address := jump_address;
					end if;

				when B_BL_GT => --(signed) x > (signed) y
					if ((in_csr_flag_z = '0') and (in_csr_flag_n = in_csr_flag_v)) then
						next_address := jump_address;
					end if;

				when B_BL_HI => --(unsigned) x > (unsigned) y
					if ((in_csr_flag_z = '0') and (in_csr_flag_c = '1')) then
						next_address := jump_address;
					end if;

				when B_BL_AL => --(x >= y) or (x < y)
					next_address := jump_address;
				when others =>
					null;

			end case;
		end if;
		out_PC_data <= next_address; -- smestamo vrednost u PC
  end process;

end architecture;
