-- data memorija
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.constants.all;

entity data_memory is

	generic 
	(
		DEFAULT_DATA_MEM_CONTENT : data_mem_array_t := (others => ZERO_ULV32_T) -- Postavlja se u startu memorija na 0. Generic je zato sto ce se load raditi iz test-a, odnosno fajla.
	);

	port 
	(
		in_clk : in std_ulogic; --clock
		in_rst : in std_ulogic; --reset
		in_address : in ulv32_t; -- adresa za ispis/upis
		in_data	: in ulv32_t; -- podaci za upis
		in_we : in bit_t; -- da li se upisuje
		out_data : out ulv32_t -- izlazni podaci
	);

end entity;

architecture rtl of data_memory is

	signal mem : data_mem_array_t := DEFAULT_DATA_MEM_CONTENT; -- memorija za podatke
  
	signal addr_reg : ulv32_t; -- adresni registar iz kojeg se vade podaci

begin

	process(in_clk, in_rst)
	begin
		if (in_rst = '1') then
			mem <= DEFAULT_DATA_MEM_CONTENT;
		elsif(rising_edge(in_clk)) then
			if(in_we = '1') then
				mem(to_integer(in_address)) <= in_data; -- Upis samo u slucaju kad je rising_edge i we = '1'
			end if;

			addr_reg <= in_address; -- adresa se smesta u registar radi ispisa koji je asinhron
		end if;
	end process;

	out_data <= mem(to_integer(addr_reg)); -- ispis

end architecture;

