-- MUX za odredjivanje sta se salje u ALU
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.constants.all;

entity alu_in_mux is
	port 
	(
		in_op_type : in type_of_instruction_t; -- tip instrukcije
		
		----OPERATOR 1
		in_rn_data : in ulv32_t; -- rn vrednost
		in_mem_data : in ulv32_t; -- mem vrednost
		in_shift_rot_data : in ulv32_t; -- s/r rn
		
		----OPERATOR_2
		in_rm_data : in ulv32_t; -- rm
		in_sign_extended_data : in ulv32_t; -- Prosirena vrednost
		
		out_operator_1 : out ulv32_t; -- izlaz za operand 1
		out_operator_2 : out ulv32_t -- izlaz za operand 2
	);
end entity;

architecture rtl of alu_in_mux is
    
begin

	with in_op_type select
		out_operator_1 <= 	in_shift_rot_data when DP_R, -- kad je DP/R salje se shift/rot vrednost
							in_rn_data when DP_I, -- kad je DP/I salje se rn
							in_mem_data when RWM, -- kad je RWM salje se vrednost iz memorije
							ZERO_ULV32_T when others; -- da se ne pravi latch
	
	with in_op_type select
		out_operator_2 <= 	in_rm_data when DP_R, -- kad je DP/R salje se rm
							in_sign_extended_data when DP_I, -- kad je DP/I salje se prosirena vrednost
							in_rm_data when RWM, -- kad je RWM salje se Rm
							ZERO_ULV32_T when others; -- da se ne pravi latch
  
end architecture;
