library ieee;
use ieee.std_logic_1164.all;

use work.constants.all;

package operand_manipulation_defs is

component shift_rot is
	port 
	(
		in_s_bit : in bit_t; -- Signed/Unsigned bit
		in_shift_or_rot : in bit_t; -- da li se shift ili rot
		in_rn_data : in ulv32_t; -- ulazni registar Rn nad kojim se radi
		in_shift_rot_dir : in u2_t; -- smer shift i rot
		in_rs_data : in ulv32_t; -- sadrzaj registra Rs koji govori za koliko se pozicija radi shift/rot
		
		out_data : out ulv32_t -- shift/rot vrednost registra Rn
	);
end component;

component sign_extender is
	port 
	(
		in_immediate : in immed_t; -- Immediate vrednost
		in_s_bit : in bit_t; -- Signed/Unsigned bit
		
		out_data : out ulv32_t -- Prosirena vrednost
	);
end component;

component alu_in_mux is
	port 
	(
		in_op_type : in type_of_instruction_t; -- tip instrukcije
		
		----OPERATOR 1
		in_rn_data : in ulv32_t; -- rn vrednost
		in_mem_data : in ulv32_t; -- mem vrednost
		in_shift_rot_data : in ulv32_t; -- s/r rn
		
		----OPERATOR_2
		in_rm_data : in ulv32_t; -- rm
		in_sign_extended_data : in ulv32_t; -- Prosirena vrednost
		
		out_operator_1 : out ulv32_t; -- izlaz za operand 1
		out_operator_2 : out ulv32_t -- izlaz za operand 2
	);
end component;

component reg_rd_mux is
	port 
	(
		in_op_type : in type_of_instruction_t; -- koji je tip instrukcije
		
		in_rd : in reg_t; -- koji je trenutni index Rd
		in_rd_data : in ulv32_t; -- vrednost rd
		in_mem_data : in ulv32_t; -- vrednost iz memorije
		in_LINK_data : in ulv32_t; -- vrednost link registra
		
		out_rd : out reg_t; -- izlaz za index
		out_data : out ulv32_t -- izlazni podaci
	);
end component;

component offset_extender is
	port 
	(
		in_offset : in offs_t; -- Offset vrednost
    
		out_data : out ulv32_t -- Prosirena vrednost
	);
end component;

component swapper is
	port 
	(
		in_swap_instruction : in bit_t; -- da li se radi swap
		in_rd_data : in ulv32_t; -- ulazna vrednost rd
		in_rm_data : in ulv32_t; -- ulazna vrednost rm
		
		out_rd_data : out ulv32_t; -- izlazna vrednost rd
		out_rm_data : out ulv32_t -- izlazna vrednost rd
	);
end component;

end package;
