-- Fajl za shiftovanje i rotiranje rn
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.constants.all;

entity shift_rot is
	port 
	(
		in_s_bit : in bit_t; -- Signed/Unsigned bit
		in_shift_or_rot : in bit_t; -- da li se shift ili rot
		in_rn_data : in ulv32_t; -- ulazni registar Rn nad kojim se radi
		in_shift_rot_dir : in u2_t; -- smer shift i rot
		in_rs_data : in ulv32_t; -- sadrzaj registra Rs koji govori za koliko se pozicija radi shift/rot
		
		out_data : out ulv32_t -- shift/rot vrednost registra Rn
	);
end entity;

architecture rtl of shift_rot is

signal result : ulv32_t; -- Rezultat shift/rot operacije

function asr(num_to_shift, ammount_to_shift : ulv32_t; signed_or_unsigned : bit_t) return ulv32_t is -- Funkcija za artimeticki shift da ne bi opterecivali dole kod
begin
	if (signed_or_unsigned = SIGNED_OP) then
		return ulv32_t(shift_right(signed(num_to_shift), to_integer(signed(ammount_to_shift)))); -- ako je signed
	else
		return ulv32_t(shift_right(unsigned(num_to_shift), to_integer(signed(ammount_to_shift)))); -- ako je unsigned
	end if;
end function;
	
begin

	out_data <= result; -- samo se interni sadrzaj proslednjuje u izlaz

	ShiftRot: process (all)
	begin
		result <= in_rn_data; -- da se ne pravi latch
		
		if (in_shift_or_rot = SHIFT_OP) then -- Gledamo da li radimo shift
		
			case in_shift_rot_dir is -- Proveravamo koja je operacija
				when S_R_LL => -- logicko ulevo
					result <= ulv32_t(shift_left(unsigned(in_rn_data), to_integer(signed(in_rs_data))));
				when S_R_LSR => -- logicko udesno
					result <= ulv32_t(shift_right(unsigned(in_rn_data), to_integer(signed(in_rs_data))));
				when S_R_ASR => -- aritmeticko udesno
					result <= asr(in_rn_data, in_rs_data, in_s_bit);
				when others => -- nema operacije
					null;
			end case;
		
		else -- Radimo rot
		
			case in_shift_rot_dir is -- Proveravamo koja je operacija
				when S_R_LL => -- logicko ulevo
					result <= ulv32_t(unsigned(in_rn_data) rol to_integer(signed(in_rs_data)));
				when (S_R_LSR) => -- logicko udesno
					result <= ulv32_t(unsigned(in_rn_data) ror to_integer(signed(in_rs_data)));
				when (S_R_ASR) => -- S_R_ASR udesno
					result <= ulv32_t(unsigned(in_rn_data) ror to_integer(signed(in_rs_data)));
				when others => -- nema operacije
					null;
			end case;
		
		end if;
	end process;

end architecture;
