--Fajl sa svim GPR registrima
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.registers_defs.all;
use work.constants.all;

entity register_file is
	generic 
	(
		DEFAULT_REG_CONTENT : reg_array_t := (others => ZERO_ULV32_T) -- Postavlja se u startu vrednost registara na 0. Generic je zato sto ce se load raditi iz test-a, odnosno fajla.
	);
	port 
	(
		in_clk : in std_logic; -- Clock processora
		in_rst : in std_logic; -- Signal koji postavlja sadrzaj registra na 0
		
		in_rn : in reg_t; -- Broj registra rn
		in_rd : in reg_t; -- Broj registra rd
		in_rm : in reg_t; -- Broj registra rm
		in_rs : in reg_t; -- Broj registra rs
		
		in_rd_data : in ulv32_t; -- Podaci koji se salju u registar rd
		in_rd_we : in bit_t; -- Upis dozvoljen u registar rd
		
		in_rm_data : in ulv32_t; -- Podaci koji se salju u registar rm
		in_rm_we : in bit_t; -- Upis dozvoljen u registar rm
		
		PC_REG_data : ulv32_t; -- Podaci za registar PC
		PC_REG_we : bit_t; -- Upis dozvoljen u registar PC
		
		out_data : out register_file_out_signals -- izlazni podaci
	);
end entity;

architecture rtl of register_file is
	
	signal registers : reg_array_t := DEFAULT_REG_CONTENT; -- niz registara
	
begin
	
	REG_RWR : process (in_clk, in_rst)
	begin
		if (in_rst = '1') then
			registers <= DEFAULT_REG_CONTENT; -- reset signal
		elsif (rising_edge(in_clk)) then
			
			if (PC_REG_we = '1') then
				registers(PC_REG_POS) <= PC_REG_data; -- ako je dozvoljen upis upisi vrednost
			end if;
			if (in_rd_we = '1') then
				registers(to_integer(in_rd)) <= in_rd_data; -- ako je dozvoljen upis upisi vrednost
			end if;
			if (in_rm_we = '1') then
				registers(to_integer(in_rm)) <= in_rm_data; -- ako je dozvoljen upis upisi vrednost
			end if;
		end if;
	end process;	
	out_data.PC_REG_data <= registers(PC_REG_POS); -- vrati trenutnu vrednost
	out_data.rn_data <= registers(to_integer(in_rn)); -- vrati trenutnu vrednost
	out_data.rd_data <= registers(to_integer(in_rd)); -- vrati trenutnu vrednost
	out_data.rm_data <= registers(to_integer(in_rm)); -- vrati trenutnu vrednost
	out_data.rs_data <= registers(to_integer(in_rs)); -- vrati trenutnu vrednost	

end architecture;
