-- Fajl za definicje registra
library ieee;
use ieee.std_logic_1164.all;

use work.constants.all;

package registers_defs is

type register_file_out_signals is record -- record podataka iz registar_file
	PC_REG_data : ulv32_t; -- Podaci iz registra PC
	
	rn_data : ulv32_t; -- Podaci iz registra rn
	rd_data : ulv32_t; -- Podaci iz registra rd
	rm_data : ulv32_t; -- Podaci iz registra rm
	rs_data : ulv32_t; -- Podaci iz registra rs
end record;

component reg is -- definicja obicnog registra (koristimo je za CSR)
	generic
	(
		DEFAULT_RESET_VALUE : ulv32_t := ZERO_ULV32_T -- mora ovako jer se IR ne sme reset na 000000.. jer je to DP/R AND operacija
	);
	port
	(
		in_clk : in std_ulogic; -- ulazni clock
		in_rst : in std_ulogic; -- reset
		in_data : in ulv32_t; -- ulazni podaci
		in_we : in bit_t; -- dozvoljen upis
		out_data : out ulv32_t -- izlazni podaci
	);
end component;

component register_file is --defincija register "kontrolera" (sadrzi unutar sebe sve registre i upisuje ili vraca podatke)
	generic 
	(
		DEFAULT_REG_CONTENT : reg_array_t := (others => ZERO_ULV32_T) -- Postavlja se u startu vrednost registara na 0. Generic je zato sto ce se load raditi iz test-a, odnosno fajla.
	);
	port 
	(
		in_clk : in std_logic; -- Clock processora
		in_rst : in std_logic; -- Signal koji postavlja sadrzaj registra na 0
		
		in_rn : in reg_t; -- Broj registra rn
		in_rd : in reg_t; -- Broj registra rd
		in_rm : in reg_t; -- Broj registra rm
		in_rs : in reg_t; -- Broj registra rs
		
		in_rd_data : in ulv32_t; -- Podaci koji se salju u registar rd
		in_rd_we : in bit_t; -- Upis dozvoljen u registar rd
		
		in_rm_data : in ulv32_t; -- Podaci koji se salju u registar rm
		in_rm_we : in bit_t; -- Upis dozvoljen u registar rm
		
		PC_REG_data : ulv32_t; -- Podaci za registar PC
		PC_REG_we : bit_t; -- Upis dozvoljen u registar PC
		
		out_data : out register_file_out_signals -- Izlazni podaci
	);
end component;

end package;
