-- Fajl za testiranje ALU jedinice
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
use std.textio.all;

use work.test.all;
use work.constants.all;
use work.alu_defs.all;

entity test_alu is

end test_alu;

architecture test of test_alu is

  -- signali koji ce se povezati sa signalima
  signal TP_in_s : alu_in_signals;

  signal TP_out_s : alu_out_signals;

begin

  Test_ALU_fn : process
    -- Variable cije vrednosti cemo da saljemo u processe 
    variable TP_in_s_var : alu_in_signals;

    variable TP_out_s_var : alu_out_signals;

    -- Broj testova i samim tim velicina 2 tabele koje koristimo
    constant number_of_tests : NATURAL := 27;

    -- Da ne bih pravio vise tabela, gde svaka sadrzi samo jedan deo instrukcije, radim append u jednu ogrmonu instrukciju pa posle radim split
    constant test_instruction_length : NATURAL := TP_in_s_var.operator_1'length + TP_in_s_var.operator_2'length + BITFLD;
    constant expectation_length : NATURAL := TP_out_s_var.result'length + BITFLD + BITFLD + BITFLD + BITFLD;

    -- Pod tip za instrukciju koja se salje
    subtype test_instruction is ulv_t(1 to test_instruction_length);

    -- Definicija niza i tabele instrukcija
    type test_instruction_array is array (natural range <>) of test_instruction;

    -- Definicija niza alu instrukcija
    type test_alu_op_array is array (natural range <>) of alu_fn_t;

          --|   opcode    |
    constant alu_op_table : test_alu_op_array(0 to number_of_tests)
    := (
        -- NOP
         0 => ALU_FN_NOP,
        -- UNSIGNED
         1 => ALU_FN_AND,
         2 => ALU_FN_AND,
         
         3 => ALU_FN_SUB_OR_CMP,
         4 => ALU_FN_SUB_OR_CMP,
         
         5 => ALU_FN_ADD,
         6 => ALU_FN_ADD,
         
         7 => ALU_FN_ADDC,
         8 => ALU_FN_ADDC,
         
         9 => ALU_FN_SUBC,
        10 => ALU_FN_SUBC,
         
        11 => ALU_FN_NOP,
        
        12 => ALU_FN_SUB_OR_CMP,
        13 => ALU_FN_SUB_OR_CMP,
        14 => ALU_FN_SUB_OR_CMP,
        
        15 => ALU_FN_MOV,
        
        16 => ALU_FN_NOT,

        -- SIGNED
        17 => ALU_FN_SUB_OR_CMP,
        18 => ALU_FN_SUB_OR_CMP,
        
        19 => ALU_FN_ADD,
        20 => ALU_FN_ADD,
        
        21 => ALU_FN_ADDC,
        22 => ALU_FN_ADDC,
        
        23 => ALU_FN_SUBC,
        24 => ALU_FN_SUBC,
        
        25 => ALU_FN_SUB_OR_CMP,
        26 => ALU_FN_SUB_OR_CMP,
        27 => ALU_FN_SUB_OR_CMP
        );

           --|     Rn       |      Rd      |   c  |
    constant instruction_table : test_instruction_array(0 to number_of_tests)
    := (
        -- RESET
         0 => to_ulv32_t(1) & to_ulv32_t(0) & '0',
        -- UNSIGNED
         1 => to_ulv32_t(1) & to_ulv32_t(0) & '0',
         2 => to_ulv32_t(1) & to_ulv32_t(1) & '0',
         
         3 => to_ulv32_t(1) & to_ulv32_t(5) & '0',
         4 => to_ulv32_t(6) & to_ulv32_t(1) & '0',
         
         5 => to_ulv32_t(1) & to_ulv32_t(1) & '0',
         6 => to_nulv32_t(-1) & to_ulv32_t(20) & '0',
         
         7 => to_ulv32_t(1) & to_ulv32_t(1) & '1',
         8 => to_nulv32_t(-1) & to_ulv32_t(20) & '1',
         
         9 => to_ulv32_t(1) & to_ulv32_t(5) & '1',
        10 => to_ulv32_t(6) & to_ulv32_t(1) & '1',
         
        11 => to_ulv32_t(0) & to_ulv32_t(3) & '0',
        
        12 => to_ulv32_t(1) & to_ulv32_t(1) & '0',
        13 => to_ulv32_t(2) & to_ulv32_t(1) & '0',
        14 => to_ulv32_t(1) & to_ulv32_t(3) & '0',
        
        15 => to_ulv32_t(7) & to_ulv32_t(3) & '0',
        
        16 => to_ulv32_t(1) & to_ulv32_t(0) & '0',

        -- SIGNED
        17 => to_nulv32_t(2147483647) & to_nulv32_t(-1) & '0',
        18 => to_nulv32_t(-2147483648) & to_nulv32_t(1) & '0',
        
        19 => to_nulv32_t(2147483647) & to_nulv32_t(2147483647) & '0',
        20 => to_nulv32_t(-1) & to_nulv32_t(-2147483648) & '0',
        
        21 => to_nulv32_t(2147483647) & to_nulv32_t(2147483647) & '1',
        22 => to_nulv32_t(-1) & to_nulv32_t(-2147483648) & '1',
        
        23 => to_nulv32_t(2147483647) & to_nulv32_t(-1) & '1',
        24 => to_nulv32_t(-2147483648) & to_nulv32_t(1) & '1',
        
        25 => to_nulv32_t(1) & to_nulv32_t(1) & '0',
        26 => to_nulv32_t(-2) & to_nulv32_t(1) & '0',
        27 => to_nulv32_t(1) & to_nulv32_t(-3) & '0'
        );

    -- Pod tip za rezultat koji ocekujemo posle izvrsenja instrukcije koja se salje
    subtype test_expectation is ulv_t(1 to expectation_length) ;

    -- Definicija niza i tabele rezultata
    type test_expectation_array is array (natural range <>) of test_expectation;

          --|     Rd        |   n  |  z  |  c  |  v |
    constant expectation_table : test_expectation_array(0 to number_of_tests)
    := (
         0 => to_ulv32_t(0)  & '0' & '0' & '0' & '0',
         -- UNSIGNED
         1 => to_ulv32_t(0)  & '0' & '1' & '0' & '0',
         2 => to_ulv32_t(1)  & '0' & '0' & '0' & '0',
         
         3 => to_nulv32_t(-4) & '1' & '0' & '1' & '0',
         4 => to_ulv32_t(5)   & '0' & '0' & '0' & '0',
         
         5 => to_ulv32_t(2)   & '0' & '0' & '0' & '0',
         6 => to_ulv32_t(19)  & '0' & '0' & '1' & '0',
         
         7 => to_ulv32_t(3)   & '0' & '0' & '0' & '0',
         8 => to_ulv32_t(20)  & '0' & '0' & '1' & '0',
         
         9 => to_nulv32_t(-5) & '1' & '0' & '1' & '0',
        10 => to_ulv32_t(4)   & '0' & '0' & '0' & '0',
         
        11 => to_ulv32_t(0)   & '0' & '0' & '0' & '0',
        
        12 => to_ulv32_t(0)   & '0' & '1' & '0' & '0',
        13 => to_ulv32_t(1)   & '0' & '0' & '0' & '0',
        14 => to_nulv32_t(-2) & '1' & '0' & '1' & '0',
        
        15 => to_ulv32_t(3)   & '0' & '0' & '0' & '0',
        
        16 => ulv32_t'(0 => '0', others => '1') & '1' & '0' & '0' & '0',
        
        
        -- SIGNED        
        17 => to_nulv32_t(-2147483648) & '1' & '0' & '1' & '1',
        18 => to_nulv32_t(2147483647) & '0' & '0' & '0' & '1',
        
        19 => to_nulv32_t(-2) & '1' & '0' & '0' & '1',
        20 => to_nulv32_t(2147483647) & '0' & '0' & '1' & '1',
        
        21 => to_nulv32_t(-1) & '1' & '0' & '0' & '1',
        22 => to_nulv32_t(-2147483648) & '1' & '0' & '1' & '1',
        
        23 => to_nulv32_t(2147483647) & '0' & '0' & '1' & '1',
        24 => to_nulv32_t(2147483646)& '0' & '0' & '0' & '1',
        
        25 => to_ulv32_t(0) & '0' & '1' & '0' & '0',
        26 => to_nulv32_t(-3) & '1' & '0' & '0' & '0',
        27 => to_nulv32_t(4) & '0' & '0' & '1' & '0'
        );

    -- variabla u koju smestamo sta se ispisiuje na konzoli/u fajl
    variable debug_line : line;

  begin
    -- Iteriramo kroz sve instrukcije tabele
    for i in 0 to number_of_tests loop
      -- Podeli instrukciju na odgovarajuce delove
      TP_in_s_var.alu_op := alu_op_table(i);
      TP_in_s_var.operator_1 := instruction_table(i)(1 to 32);
      TP_in_s_var.operator_2 := instruction_table(i)(33 to 64);
      TP_in_s_var.csr_c := to_bit_t(instruction_table(i)(65 to 65));

      -- posalji vrednosti u signale
      TP_in_s <= TP_in_s_var;

      -- Ispisi koja instrukcija se izvrsava
      write(debug_line, i, field => to_string(i)'length + 1, justified => left);
      write(debug_line, alu_fn_t'image(TP_in_s_var.alu_op), field => alu_fn_t'image(TP_in_s_var.alu_op)'length, justified => right);
      write(debug_line, to_integer(TP_in_s_var.operator_1), field => to_string(to_integer(TP_in_s_var.operator_1))'length + 1, justified => right);
      write(debug_line, to_integer(TP_in_s_var.operator_2), field => to_string(to_integer(TP_in_s_var.operator_2))'length + 1, justified => right);
      write(debug_line, to_integer(TP_in_s_var.csr_c), field => to_string(to_integer(TP_in_s_var.csr_c))'length + 1, justified => right);
      writeline(output, debug_line);
      -- mislim da nije potrebno jer writeline radi deallocate
      --deallocate(debug_line);

      -- Cekamo da signali prihvate vrednost i obrade se
      wait for half_cycle;

      -- Sta bi trebao da bude rezultat operacije
      TP_out_s_var.result := expectation_table(i)(1 to 32);
      TP_out_s_var.csr_n := to_bit_t(expectation_table(i)(33 to 33));
      TP_out_s_var.csr_z := to_bit_t(expectation_table(i)(34 to 34));
      TP_out_s_var.csr_c := to_bit_t(expectation_table(i)(35 to 35));
      TP_out_s_var.csr_v := to_bit_t(expectation_table(i)(36 to 36));

      if (verbose) then
        -- Ispisi sta smo dobili
        swrite(debug_line, "RECEIVED: ");
        write(debug_line, to_integer(TP_out_s.result), field => to_string(to_integer(TP_out_s.result))'length, justified => right);
        write(debug_line, TP_out_s.csr_n, field => to_string(TP_out_s.csr_n)'length + 1, justified => right);
        write(debug_line, TP_out_s.csr_z, field => to_string(TP_out_s.csr_z)'length + 1, justified => right);
        write(debug_line, TP_out_s.csr_c, field => to_string(TP_out_s.csr_c)'length + 1, justified => right);
        write(debug_line, TP_out_s.csr_v, field => to_string(TP_out_s.csr_v)'length + 1, justified => right);
        writeline(output, debug_line);
        -- mislim da nije potrebno jer writeline radi deallocate
        -- deallocate(debug_line);

        -- Ispisi sta ocekujemo
        swrite(debug_line, "EXPECTED: ");
        write(debug_line, to_integer(TP_out_s_var.result), field => to_string(to_integer(TP_out_s_var.result))'length, justified => right);
        write(debug_line, TP_out_s_var.csr_n, field => to_string(TP_out_s_var.csr_n)'length + 1, justified => right);
        write(debug_line, TP_out_s_var.csr_z, field => to_string(TP_out_s_var.csr_z)'length + 1, justified => right);
        write(debug_line, TP_out_s_var.csr_c, field => to_string(TP_out_s_var.csr_c)'length + 1, justified => right);
        write(debug_line, TP_out_s_var.csr_v, field => to_string(TP_out_s_var.csr_v)'length + 1, justified => right);
        writeline(output, debug_line);
        -- mislim da nije potrebno jer writeline radi deallocate
        -- deallocate(debug_line);
      end if;

      -- Prijavi ako postoje greske i koja je to tacno greska
      assert TP_out_s.result = TP_out_s_var.result 
        report "TP_out_s.result = "& to_string(to_integer(TP_out_s.result)) & "; TP_out_s_var.result = " & to_string(to_integer(TP_out_s_var.result)) severity ERROR;
      assert TP_out_s.csr_n  = TP_out_s_var.csr_n
        report "TP_out_s.csr_n = " & to_string(TP_out_s.csr_n) & "; TP_out_s_var.csr_n = " & to_string(TP_out_s_var.csr_n) severity ERROR;
      assert TP_out_s.csr_z  = TP_out_s_var.csr_z
        report "TP_out_s.csr_z = " & to_string(TP_out_s.csr_z) & "; TP_out_s_var.csr_z = " & to_string(TP_out_s_var.csr_z) severity ERROR;
      assert TP_out_s.csr_c  = TP_out_s_var.csr_c
        report "TP_out_s.csr_c = " & to_string(TP_out_s.csr_c) & "; TP_out_s_var.csr_c = " & to_string(TP_out_s_var.csr_c) severity ERROR;
      assert TP_out_s.csr_v  = TP_out_s_var.csr_v
        report "TP_out_s.csr_v = " & to_string(TP_out_s.csr_v) & "; TP_out_s_var.csr_v = " & to_string(TP_out_s_var.csr_v) severity ERROR;

    end loop;
  end process Test_ALU_fn;

	-- mapiranje test portova
	P_COMP :  entity work.alu(alu_adders_arch)
	generic map
	(
		ADDER_TYPE => RIPPLE_NO_ZERO_PROPAGATION
	)
	port map 
	(    
		in_s => TP_in_s,

		out_s => TP_out_s
	);

end architecture test;
