-- Fajl za testiranje Decoder jedinice
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
use std.textio.all;

use work.test.all;
use work.constants.all;
use work.control_unit_defs.all;

entity test_decoder is

end test_decoder;

architecture test of test_decoder is

  -- signali koji ce se povezati sa signalima  
  signal TP_in_instruction : ulv32_t;
  
  signal TP_out_s : decoder_out_signals;

begin

  test_sign_extender_fn : process
    -- Variable cije vrednosti cemo da saljemo u processe 
    variable TP_in_instruction_var : ulv32_t;

		variable TP_out_s_var : decoder_out_signals;

    -- Broj testova i samim tim velicina 2 tabele koje koristimo
    constant number_of_tests : NATURAL := 6;

    -- Da ne bih pravio vise tabela, gde svaka sadrzi samo jedan deo instrukcije, radim append u jednu ogrmonu instrukciju pa posle radim split
    constant test_instruction_length : NATURAL := TP_in_instruction_var'length;
    constant expectation_length : NATURAL := TP_in_instruction_var'length - 1;

    -- Pod tip za instrukciju koja se salje
    subtype test_instruction is ulv_t(1 to test_instruction_length) ;

    -- Definicija niza i tabele instrukcija
    type test_instruction_array is array (natural range <>) of test_instruction;

          --|                                              instruction                                                 |
    constant instruction_table : test_instruction_array(0 to number_of_tests)
    := (
         0 => OP_TYPE_DP_R & OP_CODE_AND & "0000" & "0001" & UNSIGNED_OP & "0010" & "0011" & ROT_OP & S_R_LL & "00000",
         1 => OP_TYPE_DP_R & OP_CODE_AND & "0000" & "0001" & UNSIGNED_OP & "0010" & "0011" & ROT_OP & S_R_LL & "00000",
         2 => OP_TYPE_DP_I & OP_CODE_SUB & "0000" & "0001" & UNSIGNED_OP & immed_t'(1 => '0', others => '1'),
         
         3 => OP_TYPE_L_S & L_S_S & "000" & "0000" & "0001" & "00000000000000000",
         4 => OP_TYPE_RWM & OP_CODE_ADD & "0000" & "0001" & UNSIGNED_OP & "0010" & "000000000000",
         5 => OP_TYPE_B_BL & B_BL_EQ & B_BL_BL & offs_t'(1 => '0', others => '1'),
         6 => OP_TYPE_STOP & "00000000000000000000000000000"
        );

    -- Pod tip za rezultat koji ocekujemo posle izvrsenja instrukcije koja se salje
    subtype test_expectation is ulv_t(expectation_length downto 0) ;

    -- Definicija niza i tabele rezultata
    type test_expectation_array is array (natural range <>) of test_expectation;

          --|          result          |
    constant expectation_table : test_expectation_array(0 to number_of_tests)
    := (
         0 => OP_TYPE_DP_R & OP_CODE_AND & "0000000000000000000000000",
         1 => OP_TYPE_DP_R & OP_CODE_AND & "0000" & "0001" & UNSIGNED_OP & "0010" & "0011" & ROT_OP & S_R_LL & "00000",
         2 => OP_TYPE_DP_I & OP_CODE_SUB & "0000" & "0001" & UNSIGNED_OP & immed_t'(1 => '0', others => '1'),
         
         3 => OP_TYPE_L_S & L_S_S & "000" & "0000" & "0001" & "00000000000000000",
         4 => OP_TYPE_RWM & OP_CODE_ADD & "0000" & "0001" & UNSIGNED_OP & "0010" & "000000000000",
         5 => OP_TYPE_B_BL & B_BL_EQ & B_BL_BL & offs_t'(1 => '0', others => '1'),
         6 => OP_TYPE_STOP & "00000000000000000000000000000"
        );

    -- variabla u koju smestamo sta se ispisiuje na konzoli/u fajl
    variable debug_line : line;

  begin
    -- Iteriramo kroz sve instrukcije tabele
    for i in 0 to number_of_tests loop
      -- Podeli instrukciju na odgovarajuce delove
      TP_in_instruction_var := instruction_table(i)(1 to 32);

      -- posalji vrednosti u signale
      TP_in_instruction <= TP_in_instruction_var;

      -- Ispisi koja instrukcija se izvrsava
      write(debug_line, i, field => to_string(i)'length + 1, justified => left);
      write(debug_line, TP_in_instruction, field => TP_in_instruction'length + 1, justified => right);
      writeline(output, debug_line);
      -- mislim da nije potrebno jer writeline radi deallocate
      --deallocate(debug_line);

      -- Cekamo da signali prihvate vrednost i obrade se
      wait for half_cycle;

      -- Sta bi trebao da bude rezultat operacije      
      TP_out_s_var.op_type := expectation_table(i)(31 downto 29);
      TP_out_s_var.op_code := expectation_table(i)(28 downto 25);
      TP_out_s_var.rn := expectation_table(i)(24 downto 21);
      TP_out_s_var.rd := expectation_table(i)(20 downto 17);
      TP_out_s_var.s_bit := to_bit_t(expectation_table(i)(16 downto 16));
      TP_out_s_var.rm := expectation_table(i)(15 downto 12);
      TP_out_s_var.rs := expectation_table(i)(11 downto 8);
      TP_out_s_var.shift_or_rot := to_bit_t(expectation_table(i)(7 downto 7));
      TP_out_s_var.shift_rot_dir := expectation_table(i)(6 downto 5);
      TP_out_s_var.immediate := expectation_table(i)(15 downto 0);
      TP_out_s_var.load_or_store := to_bit_t(expectation_table(i)(28 downto 28));
      TP_out_s_var.jump_condition := expectation_table(i)(28 downto 27);
      TP_out_s_var.branch_or_branch_and_link := to_bit_t(expectation_table(i)(26 downto 26));
      TP_out_s_var.jump_offset := expectation_table(i)(25 downto 0);

      if (verbose) then
        -- Ispisi sta smo dobili
        swrite(debug_line, "RECEIVED: ");
        write(debug_line, trim_string(op_type_instructions_name_table(to_integer(TP_out_s_var.op_type))), field => trim_string(op_type_instructions_name_table(to_integer(TP_out_s_var.op_type)))'length, justified => right);
        write(debug_line, trim_string(alu_instructions_name_table(to_integer(TP_out_s_var.op_code))), field => trim_string(alu_instructions_name_table(to_integer(TP_out_s_var.op_code)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rn), field => to_string(to_integer(TP_out_s_var.rn))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rd), field => to_string(to_integer(TP_out_s_var.rd))'length + 1, justified => right);
        write(debug_line, trim_string(signed_unsigned_name_table(to_integer(TP_out_s_var.s_bit))), field => trim_string(signed_unsigned_name_table(to_integer(TP_out_s_var.s_bit)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rm), field => to_string(to_integer(TP_out_s_var.rm))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rs), field => to_string(to_integer(TP_out_s_var.rs))'length + 1, justified => right);
        write(debug_line, trim_string(shift_rot_name_table(to_integer(TP_out_s_var.shift_or_rot))), field => trim_string(shift_rot_name_table(to_integer(TP_out_s_var.shift_or_rot)))'length + 1, justified => right);
        write(debug_line, trim_string(shift_dir_name_table(to_integer(TP_out_s_var.shift_rot_dir))), field => trim_string(shift_dir_name_table(to_integer(TP_out_s_var.shift_rot_dir)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.immediate), field => to_string(to_integer(TP_out_s_var.immediate))'length + 1, justified => right);
        write(debug_line, trim_string(load_store_name_table(to_integer(TP_out_s_var.load_or_store))), field => trim_string(load_store_name_table(to_integer(TP_out_s_var.load_or_store)))'length + 1, justified => right);
        write(debug_line, trim_string(jump_condition_name_table(to_integer(TP_out_s_var.jump_condition))), field => trim_string(jump_condition_name_table(to_integer(TP_out_s_var.jump_condition)))'length + 1, justified => right);
        write(debug_line, trim_string(branch_name_table(to_integer(TP_out_s_var.branch_or_branch_and_link))), field => trim_string(branch_name_table(to_integer(TP_out_s_var.branch_or_branch_and_link)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.jump_offset), field => to_string(to_integer(TP_out_s_var.jump_offset))'length + 1, justified => right);
        
        writeline(output, debug_line);
        -- mislim da nije potrebno jer writeline radi deallocate
        -- deallocate(debug_line);

        -- Ispisi sta ocekujemo
        swrite(debug_line, "EXPECTED: ");
        write(debug_line, trim_string(op_type_instructions_name_table(to_integer(TP_out_s_var.op_type))), field => trim_string(op_type_instructions_name_table(to_integer(TP_out_s_var.op_type)))'length, justified => right);
        write(debug_line, trim_string(alu_instructions_name_table(to_integer(TP_out_s_var.op_code))), field => trim_string(alu_instructions_name_table(to_integer(TP_out_s_var.op_code)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rn), field => to_string(to_integer(TP_out_s_var.rn))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rd), field => to_string(to_integer(TP_out_s_var.rd))'length + 1, justified => right);
        write(debug_line, trim_string(signed_unsigned_name_table(to_integer(TP_out_s_var.s_bit))), field => trim_string(signed_unsigned_name_table(to_integer(TP_out_s_var.s_bit)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rm), field => to_string(to_integer(TP_out_s_var.rm))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.rs), field => to_string(to_integer(TP_out_s_var.rs))'length + 1, justified => right);
        write(debug_line, trim_string(shift_rot_name_table(to_integer(TP_out_s_var.shift_or_rot))), field => trim_string(shift_rot_name_table(to_integer(TP_out_s_var.shift_or_rot)))'length + 1, justified => right);
        write(debug_line, trim_string(shift_dir_name_table(to_integer(TP_out_s_var.shift_rot_dir))), field => trim_string(shift_dir_name_table(to_integer(TP_out_s_var.shift_rot_dir)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.immediate), field => to_string(to_integer(TP_out_s_var.immediate))'length + 1, justified => right);
        write(debug_line, trim_string(load_store_name_table(to_integer(TP_out_s_var.load_or_store))), field => trim_string(load_store_name_table(to_integer(TP_out_s_var.load_or_store)))'length + 1, justified => right);
        write(debug_line, trim_string(jump_condition_name_table(to_integer(TP_out_s_var.jump_condition))), field => trim_string(jump_condition_name_table(to_integer(TP_out_s_var.jump_condition)))'length + 1, justified => right);
        write(debug_line, trim_string(branch_name_table(to_integer(TP_out_s_var.branch_or_branch_and_link))), field => trim_string(branch_name_table(to_integer(TP_out_s_var.branch_or_branch_and_link)))'length + 1, justified => right);
        write(debug_line, to_integer(TP_out_s_var.jump_offset), field => to_string(to_integer(TP_out_s_var.jump_offset))'length + 1, justified => right);
        writeline(output, debug_line);
        -- mislim da nije potrebno jer writeline radi deallocate
        -- deallocate(debug_line);
      end if;

      -- Prijavi ako postoje greske i koja je to tacno greska
      assert TP_out_s_var.op_type = TP_out_s_var.op_type 
        report "TP_out_s_var.op_type = "& to_string(to_integer(TP_out_s_var.op_type)) & "; TP_out_s_var.op_type = " & to_string(to_integer(TP_out_s_var.op_type)) severity ERROR;
      assert TP_out_s_var.op_code = TP_out_s_var.op_code 
        report "TP_out_s_var.op_code = "& to_string(to_integer(TP_out_s_var.op_code)) & "; TP_out_s_var.op_code = " & to_string(to_integer(TP_out_s_var.op_code)) severity ERROR;
      assert TP_out_s_var.rn = TP_out_s_var.rn 
        report "TP_out_s_var.rn = "& to_string(to_integer(TP_out_s_var.rn)) & "; TP_out_s_var.rn = " & to_string(to_integer(TP_out_s_var.rn)) severity ERROR;
      assert TP_out_s_var.rd = TP_out_s_var.rd 
        report "TP_out_s_var.rd = "& to_string(to_integer(TP_out_s_var.rd)) & "; TP_out_s_var.rd = " & to_string(to_integer(TP_out_s_var.rd)) severity ERROR;
      assert TP_out_s_var.s_bit  = TP_out_s_var.s_bit
        report "TP_out_s_var.s_bit = " & to_string(TP_out_s_var.s_bit) & "; TP_out_s_var.s_bit = " & to_string(TP_out_s_var.s_bit) severity ERROR;
      assert TP_out_s_var.rm = TP_out_s_var.rm 
        report "TP_out_s_var.rm = "& to_string(to_integer(TP_out_s_var.rm)) & "; TP_out_s_var.rm = " & to_string(to_integer(TP_out_s_var.rm)) severity ERROR;
      assert TP_out_s_var.rs = TP_out_s_var.rs 
        report "TP_out_s_var.rs = "& to_string(to_integer(TP_out_s_var.rs)) & "; TP_out_s_var.rs = " & to_string(to_integer(TP_out_s_var.rs)) severity ERROR;
      assert TP_out_s_var.shift_or_rot  = TP_out_s_var.shift_or_rot
        report "TP_out_s_var.shift_or_rot = " & to_string(TP_out_s_var.shift_or_rot) & "; TP_out_s_var.shift_or_rot = " & to_string(TP_out_s_var.shift_or_rot) severity ERROR;
      assert TP_out_s_var.shift_rot_dir = TP_out_s_var.shift_rot_dir 
        report "TP_out_s_var.shift_rot_dir = "& to_string(TP_out_s_var.shift_rot_dir) & "; TP_out_s_var.shift_rot_dir = " & to_string(TP_out_s_var.shift_rot_dir) severity ERROR;
      assert TP_out_s_var.immediate = TP_out_s_var.immediate 
        report "TP_out_s_var.immediate = "& to_string(to_integer(TP_out_s_var.immediate)) & "; TP_out_s_var.immediate = " & to_string(to_integer(TP_out_s_var.immediate)) severity ERROR;
      assert TP_out_s_var.load_or_store  = TP_out_s_var.load_or_store
        report "TP_out_s_var.load_or_store = " & to_string(TP_out_s_var.load_or_store) & "; TP_out_s_var.load_or_store = " & to_string(TP_out_s_var.load_or_store) severity ERROR;
      assert TP_out_s_var.jump_condition = TP_out_s_var.jump_condition 
        report "TP_out_s_var.jump_condition = "& to_string(TP_out_s_var.jump_condition) & "; TP_out_s_var.jump_condition = " & to_string(TP_out_s_var.jump_condition) severity ERROR;
      assert TP_out_s_var.branch_or_branch_and_link  = TP_out_s_var.branch_or_branch_and_link
        report "TP_out_s_var.branch_or_branch_and_link = " & to_string(TP_out_s_var.branch_or_branch_and_link) & "; TP_out_s_var.branch_or_branch_and_link = " & to_string(TP_out_s_var.branch_or_branch_and_link) severity ERROR;
      assert TP_out_s_var.jump_offset = TP_out_s_var.jump_offset 
        report "TP_out_s_var.jump_offset = "& to_string(to_integer(TP_out_s_var.jump_offset)) & "; TP_out_s_var.jump_offset = " & to_string(to_integer(TP_out_s_var.jump_offset)) severity ERROR;

    end loop;
  end process test_sign_extender_fn;

    -- mapiranje test portova
		P_COMP : Decoder
    port map 
    (
      in_instruction => TP_in_instruction,
      
			out_s => TP_out_s
    );

end architecture test;
