-- Fajl za testiranje Processora
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
use std.textio.all;

use work.test.all;
use work.constants.all;

entity test_processor is

end test_processor;

architecture test of test_processor is

	type mem_array_t is array(natural range <>) of ulv32_t;

  impure function load_mem (mem_file_name : in string; size : in natural)
                            return mem_array_t is
    file mem_file : text open read_mode is mem_file_name;
		variable i : natural;
    variable L : line;
    variable data : ulv32_t;
    variable mem : mem_array_t(0 to size);
  begin
		i := 0;

		loop_until_eof:
		while not endfile(mem_file) loop
			if (i <= size) then
				readline(mem_file, L);
      	read(L, data);
      	mem(i) := ulv32_t(data);
			else
				exit loop_until_eof;
			end if;
			i := i + 1;
		end loop;

		while (i <= size) loop
      mem(i) := ZERO_ULV32_T;
			i := i + 1;
		end loop;

    return mem;
  end function;
  
  -- signali koji ce se povezati 
  signal in_TP_clk : std_logic := '0';
  signal in_TP_rst : std_logic := '0';
  
  component Processor is
    generic 
    (
      DEFAULT_INST_MEM_CONTENT : inst_mem_array_t := (others => ZERO_ULV32_T);
      DEFAULT_DATA_MEM_CONTENT : data_mem_array_t := (others => ZERO_ULV32_T);
      DEFAULT_REG_CONTENT : reg_array_t := (others => ZERO_ULV32_T);
      PRINT_COMMANDS : boolean := false
    );
    port 
    (
      signal in_clk : in std_logic; -- Clock processora
      signal in_rst : in std_logic -- Reset signal koji resetuje jedinicu
    );
  end component Processor;

begin
  
  reset_gen : in_TP_rst <= '0',
                              '1' after   5 ns,
                              '0' after  25 ns;

  clk_gen : process
  begin
    in_TP_clk <= '0';
    wait for 10 ns;
    loop
      in_TP_clk <= '1', '0' after 5 ns;
      wait for 10 ns;
    end loop;
  end process clk_gen;

  RunTP : process (in_TP_clk)
  begin
  end process RunTP;

  P_COMP : Processor
    generic map 
    (
      DEFAULT_INST_MEM_CONTENT => inst_mem_array_t(load_mem("init_im.mem", inst_mem_array_t'high)),
--      (
--             
--             -- 0 => OP_TYPE_DP_I &   OP_CODE_SUB         & R0 & R0 & UNSIGNED_OP      &  "0000000000000001",
--             -- 1 =>  OP_TYPE_L_S &         L_S_S & "000" & R1 & R0                    & "00000000000000000",
--             -- 2 => OP_TYPE_DP_I &   OP_CODE_ADD         & R0 & R0 & UNSIGNED_OP      &  "0000000000000001",
--             -- 3 =>  OP_TYPE_RWM &   OP_CODE_ADD         & R0 & R1 & UNSIGNED_OP & R2 &      "000000000000",
--             -- 4 => OP_TYPE_B_BL &     B_BL_AL & B_BL_BL & offs_t(to_unsigned(3, offs_t'length)),
--             -- 5 => OP_TYPE_DP_I &   OP_CODE_SUB         & R0 & R0 & UNSIGNED_OP      &  "0000000000000001",
--             -- 6 => OP_TYPE_DP_I &   OP_CODE_NOT         & R0 & R0 & UNSIGNED_OP      &  "0000000000000001",
--             -- 7 =>  OP_TYPE_L_S &         L_S_L & "000" & R1 & R0                    & "00000000000000000",
--             -- 8 =>  OP_TYPE_L_S &         L_S_S & "000" & R1 & R0                    & "00000000000000000",
--             -- 9 =>  OP_TYPE_RWM &   OP_CODE_CMP         & R0 & R1 & UNSIGNED_OP & R2 &      "000000000000",
----             -----------------------------------------|  RN |  RD |
----              0 => OP_TYPE_DP_I &   OP_CODE_ADD         & R0  & R0  & UNSIGNED_OP      &  "0000000000000001", --r0 ?= 0 + 1 ?= 1   , r0 ?=  1, r1 ?=  0, r2 ?=  0, r15 ?=  1, m0 ?= 0
----              1 => OP_TYPE_DP_I &   OP_CODE_ADD         & R0  & R0  & UNSIGNED_OP      &  "0000000000000001", --r0 ?= 1 + 1 ?= 2   , r0 ?=  2, r1 ?=  0, r2 ?=  0, r15 ?=  2, m0 ?= 0
----              2 =>  OP_TYPE_L_S &         L_S_S & "000" & R1  & R0                     & "00000000000000000", --meme0 ?= 2        , r0 ?=  2, r1 ?=  0, r2 ?=  0, r15 ?=  3, m0 ?= 2
----              3 => OP_TYPE_DP_I &   OP_CODE_ADD         & R0  & R0  & UNSIGNED_OP      &  "0000000000000001", --r0 ?= 2 + 1 ?= 3   , r0 ?=  3, r1 ?=  0, r2 ?=  0, r15 ?=  4, m0 ?= 2
----              4 => OP_TYPE_DP_I &   OP_CODE_ADD         & R1  & R1  & UNSIGNED_OP      &  "0000000000000111", --r1 ?= 0 + 7 ?= 7   , r0 ?=  3, r1 ?=  7, r2 ?=  0, r15 ?=  5, m0 ?= 2
----              5 => OP_TYPE_DP_I &   OP_CODE_MOV         & R1  & R15 & UNSIGNED_OP      &  "0000000000000011", --r15 ?=  3, r1 ?=  0, r0 ?=  3, r1 ?=  7, r2 ?=  0, r15 ?=  3, m0 ?= 2
----              6 =>  OP_TYPE_RWM &   OP_CODE_ADD         & R2  & R0  & UNSIGNED_OP & R0 &      "000000000000", --r0 ?= 2, mem0 ?=  4, r0 ?=  2, r1 ?=  3, r2 ?=  0, r15 ?=  4, m0 ?= 4
----              7 => OP_TYPE_B_BL &     B_BL_AL &  B_BL_B & offs_t(to_unsigned(3, offs_t'length)),
----              8 => OP_TYPE_DP_I &   OP_CODE_SUB         & R0  & R0  & UNSIGNED_OP      &  "0000000000000001",
----              9 => OP_TYPE_DP_I &   OP_CODE_NOT         & R0  & R0  & UNSIGNED_OP      &  "0000000000000001",
----             10 =>  OP_TYPE_L_S &         L_S_L & "000" & R1  & R0                     & "00000000000000000",
----             11 =>  OP_TYPE_L_S &         L_S_S & "000" & R1  & R0                     & "00000000000000000",
----             12 =>  OP_TYPE_RWM &   OP_CODE_CMP         & R0  & R1  & UNSIGNED_OP & R2 &      "000000000000",
--            
--            16 => "00000100110011000110000000011111",
--            17 => "01010000110011000000000000000000",
--            18 => "01010000110010100000000000000000",
--            19 => "00100100101010100000000000000001",
--            20 => "10001011111111111111111111111110",
--            21 => "01000000110010100000000000000000",
--            22 => "10100000000000000000000000000000",
--        others =>  OP_TYPE_STOP &  OP_CODE_NOP & "0000000000000000000000000"
--      ),
      
      DEFAULT_DATA_MEM_CONTENT => data_mem_array_t(load_mem("init_dm.mem", data_mem_array_t'high)),
--      (
--             0 => to_ulv32_t(256),
--             256 => to_ulv32_t(128),
--        others => ZERO_ULV32_T
--      ),
      
      DEFAULT_REG_CONTENT => reg_array_t(load_mem("init_regs.mem", reg_array_t'high)),
--      (
--             to_integer(PC_REG) => to_ulv32_t(16),
--        others => ZERO_ULV32_T
--      ),
      
      PRINT_COMMANDS => true
    )
    port map 
    (
      in_clk => in_TP_clk,
      in_rst => in_TP_rst
    );

end test;
