-- Fajl za testiranje Sign Extender jedinice
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
use std.textio.all;

use work.test.all;
use work.constants.all;
use work.operand_manipulation_defs.all;

entity test_sign_extender is

end test_sign_extender;

architecture test of test_sign_extender is

  -- signali koji ce se povezati
  signal TP_in_immediate : immed_t;
  signal TP_in_s_bit : bit_t;
  
  signal TP_out_data : ulv32_t;

begin

  test_sign_extender_fn : process
    -- Variable cije vrednosti cemo da saljemo u processe 
    variable TP_in_immediate_var : immed_t;
    variable TP_in_s_bit_var : bit_t;
    
    variable TP_out_data_var : ulv32_t;

    -- Broj testova i samim tim velicina 2 tabele koje koristimo
    constant number_of_tests : NATURAL := 4;

    -- Da ne bih pravio vise tabela, gde svaka sadrzi samo jedan deo instrukcije, radim append u jednu ogrmonu instrukciju pa posle radim split
    constant test_instruction_length : NATURAL := TP_in_immediate_var'length + BITFLD;
    constant expectation_length : NATURAL := TP_out_data_var'length;

    -- Pod tip za instrukciju koja se salje
    subtype test_instruction is ulv_t(1 to test_instruction_length) ;

    -- Definicija niza i tabele instrukcija
    type test_instruction_array is array (natural range <>) of test_instruction;

          --|             immediate            |      S      |
    constant instruction_table : test_instruction_array(0 to number_of_tests)
    := (
         0 => immed_t'(15 => '0', others => '1') & UNSIGNED_OP,
         1 => immed_t'(15 => '0', others => '1') & UNSIGNED_OP,
         2 => immed_t'(15 => '1', others => '1') & UNSIGNED_OP,
         
         3 => immed_t'(15 => '0', others => '1') &   SIGNED_OP,
         4 => immed_t'(15 => '1', others => '1') &   SIGNED_OP
        );

    -- Pod tip za rezultat koji ocekujemo posle izvrsenja instrukcije koja se salje
    subtype test_expectation is ulv_t(1 to expectation_length) ;

    -- Definicija niza i tabele rezultata
    type test_expectation_array is array (natural range <>) of test_expectation;

          --|       RESULT       |
    constant expectation_table : test_expectation_array(0 to number_of_tests)
    := (
         0 => to_ulv32_t(32767),
         1 => ulv32_t'(31 downto 15 => '0', others => '1'),
         2 => ulv32_t'(31 downto 16 => '0', others => '1'),
         
         3 => ulv32_t'(31 downto 15 => '0', others => '1'),
         4 => ulv32_t'(31 downto 15 => '1', others => '1')
        );

    -- variabla u koju smestamo sta se ispisiuje na konzoli/u fajl
    variable debug_line : line;

  begin
    -- Iteriramo kroz sve instrukcije tabele
    for i in 0 to number_of_tests loop
      -- Podeli instrukciju na odgovarajuce delove
      TP_in_immediate_var := instruction_table(i)(1 to 16);
      TP_in_s_bit_var := to_bit_t(instruction_table(i)(17 to 17));

      -- posalji vrednosti u signale
      TP_in_immediate <= TP_in_immediate_var;
      TP_in_s_bit <= TP_in_s_bit_var;

      -- Ispisi koja instrukcija se izvrsava
      write(debug_line, i, field => to_string(i)'length + 1, justified => left);
      write(debug_line, TP_in_immediate_var, field => TP_in_immediate_var'length + 1, justified => right);
      write(debug_line, trim_string(signed_unsigned_name_table(to_integer(TP_in_s_bit_var))), field => trim_string(signed_unsigned_name_table(to_integer(TP_in_s_bit_var)))'length + 1, justified => right);
      writeline(output, debug_line);
      -- mislim da nije potrebno jer writeline radi deallocate
      --deallocate(debug_line);

      -- Cekamo da signali prihvate vrednost i obrade se
      wait for half_cycle;

      -- Sta bi trebao da bude rezultat operacije
      TP_out_data_var := expectation_table(i);

      if (verbose) then
        -- Ispisi sta smo dobili
        swrite(debug_line, "RECEIVED: ");
        write(debug_line, to_integer(TP_out_data), field => to_string(to_integer(TP_out_data))'length, justified => right);
        writeline(output, debug_line);
        -- mislim da nije potrebno jer writeline radi deallocate
        -- deallocate(debug_line);

        -- Ispisi sta ocekujemo     
        swrite(debug_line, "EXPECTED: ");
        write(debug_line, to_integer(TP_out_data_var), field => to_string(to_integer(TP_out_data_var))'length, justified => right);
        writeline(output, debug_line);
        -- mislim da nije potrebno jer writeline radi deallocate
        -- deallocate(debug_line);
      end if;

      -- Prijavi ako postoje greske i koja je to tacno greska
      assert TP_out_data ?= TP_out_data_var 
        report "TP_out_data_var ?= "& to_string(to_integer(TP_out_data_var)) & "; TP_out_data ?= " & to_string(to_integer(TP_out_data)) severity ERROR;

    end loop;
  end process test_sign_extender_fn;

    -- mapiranje test portova
		P_COMP : sign_extender
    port map 
    (    
      in_immediate => TP_in_immediate,
      in_s_bit => TP_in_s_bit,
      
      out_data => TP_out_data
    );

end architecture test;
