import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Assembler {

	private static Path current_path = Paths.get(""); // trenutni direktorijum
	private static Path load_file_path = Paths.get(current_path + "prog.asm"); // fajl iz kojeg se cita
	private static Path save_file_path = Paths.get(current_path + "inst.mem"); // fajl u koji se upisuje

	//broj parametara za svaku instrukciju
	private static final short DP_R = 8;
	private static final short DP_I = 5;
	private static final short L_S = 3;
	private static final short RWM = 5;
	private static final short B_BL = 3;
	private static final short STOP = 1;

	//LUT tabela sa konverzijama
	private static final Map<String, String> lut = new HashMap<String, String>() {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		{
			put("AND", "0000");
			put("SUB", "0010");
			put("ADD", "0100");
			put("ADC", "0101");
			put("SBC", "0110");
			put("SWAP", "1000");
			put("CMP", "1010");
			put("MOV", "1101");
			put("NOT", "1111");

			put("EQ", "00");
			put("GT", "01");
			put("HI", "10");
			put("AL", "11");

			put("LL", "00");
			put("LSR", "01");
			put("ASR", "10");
			put("-", "11");

			//trebalo bi da bude samo 1 ili 0 ali posto iza njih idu 3 prazna bita ovako je lakse
			put("LOAD", "1000");
			put("STORE", "0000");

			put("B", "0");
			put("BL", "1");

			put("STOP", "101");

			put("U", "0");
			put("S", "1");

			put("SHF", "0");
			put("ROT", "1");

			put("$R0", "0000");
			put("$R1", "0001");
			put("$R2", "0010");
			put("$R3", "0011");
			put("$R4", "0100");
			put("$R5", "0101");
			put("$R6", "0110");
			put("$R7", "0111");
			put("$R8", "1000");
			put("$R9", "1001");
			put("$R10", "1010");
			put("$R11", "1011");
			put("$R12", "1100");
			put("$R13", "1101");
			put("$R14", "1110");
			put("$R15", "1111");
		}
	};

	private static StringBuilder sb = new StringBuilder();

	// binarno prosirivanje na odredjenu velicinu
	public static String expand(int s, int len) {
		return String.format("%0" + (len - Integer.toBinaryString(s).length())  + "d%s", 0, Integer.toBinaryString(s));
	}

	public static void main(String[] args) {
		// ucitavamo sve linije
		List<String> commands = null;
		try {
			commands = Files.readAllLines(load_file_path,
					Charset.forName("utf-8"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		}
		
		List<String> binary_commands = new ArrayList<>();

		for (String command : commands) {
			// cistimo nepotrebne space-ove
			command = command.trim();
			// da olaksamo sve prevodimo u velika slova
			command = command.toUpperCase();
			
			// nisu nam potrebni prazne linije
			if (command.length() > 0) {
				
				//delimo po <space>
				String[] params = command.split(" ");
				
				// automatski trazimo u LUT i ako pronadjemo parametar stavljamo u string
				// ovo za null je ako se nadju brojevi da ne bi upisivao null
				// i brojevi su u svakom slucaju poslednji parameteri tako da se rade na drugi nacin
				for (int i = 0; i < params.length; i++) {
					sb.append(lut.get(params[i]) != null ? lut.get(params[i])
							: "");
				}

				// po brojo parametara zakljucijemo s cim radimo
				switch (params.length) {
				case DP_R:
					// na pocetak stavljamo
					sb.insert(0, "000");
					// na kraj stavljamo da bi popunili 32 bita
					sb.append(expand(0, 4));
					break;
				case L_S | B_BL:
					if (params[0].equals("LOAD") || params[0].equals("STORE")) {
						// na pocetak stavljamo
						sb.insert(0, "010");
						// na kraj stavljamo da bi popunili 32 bita
						sb.append(expand(0, 16));
					} else {
						// na pocetak stavljamo
						sb.insert(0, "100");
						// radimo konverziju iz dec u bin i prosirujemo
						sb.append(expand(
								Integer.parseInt(params[2].substring(1)), 25));
					}
					break;
				case STOP:
					// na kraj stavljamo da bi popunili 32 bita
					sb.append(expand(0, 28));
					break;
				case DP_I | RWM:
					if (params[4].contains("#")) // DP I
					{
						// na pocetak stavljamo
						sb.insert(0, "001");
						// radimo konverziju iz dec u bin i prosirujemo
						sb.append(expand(
								Integer.parseInt(params[4].substring(1)), 15));
					} else {
						// na pocetak stavljamo
						sb.insert(0, "101");
						// na kraj stavljamo da bi popunili 32 bita
						sb.append(expand(0, 11));
					}
					break;

				default:
					System.exit(-5);
					break;
				}

				// cisto da nam se ispise
				System.out.print(sb.toString() + " - ");
				System.out.println(command);

				// dodajemo u listu komandi koju cuvamo
				binary_commands.add(sb.toString());
				//cistimo StringBuilder
				sb.delete(0, sb.length());
			}
			// cuvamo u fajl
			try {
				Files.write(save_file_path, binary_commands, Charset.forName("utf-8"));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

}
